#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
    /// </summary>
    [Description("Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support")]
    public class BLine : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int length1 = 35; // Default setting for Length1
            private int length2 = 10; // Default setting for Length2
			private int length3 = 1; //Defualt seeting for Length3
            private int overBought = 80; // Default setting for OverBought
            private int overSold = 20; // Default setting for OverSold
		
			private bool			was_obos = false;
        // User defined variables (add any user defined variables below)
		
		private DataSeries k_fast;
		private DataSeries k_full;
		private DataSeries d_full;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "MidColor"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "OBOSColor"));
            Add(new Line(Color.FromKnownColor(KnownColor.DarkOliveGreen), 80, "OverBought"));
            Add(new Line(Color.FromKnownColor(KnownColor.Khaki), 20, "OverSold"));
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= true;
			
			k_fast = new DataSeries(this);
			k_full = new DataSeries(this);
			d_full = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
//-------------------------------------------------------------------------------------------------------
//Translated from (EasyLanguage):
//Input: Price(Close), Length1(35), Length2(10), Length3(1), OverBought(80), OverSold(20),
//MidColor(White),OBOSColor(Red);
//Variables: KFast(0), KFull(0), DFull(0), LL(O), HH(0);
//LL = Lowest((High+Low)/2, Length1);
//HH = Highest((High+Low)/2, Length1);
//KFast = 100 * IFF(HH-LL=0,0,(((High+Low)/2) - LL)/(HH- LL));
//KFull = Average(KFast, Length2);
//DFull = Average(KFull, Length3);
//If KFull <= 80 and KFull >= 20 then 
//Plot1(KFull,"Stoc",MidColor,Default,2) else
//Plot1(KFull,"Stoc",OBOSColor,Default,2);
//Plot3(OverBought, "OverBought");
//Plot4(OverSold, "OverSold");
//-------------------------------------------------------------------------------------------------------
			
			if(CurrentBar < length1) return;
			
			
			double ll = MIN(Median, Length1)[0];
			
			double hh = MAX(Median, Length1)[0];
			
			k_fast.Set(100 * (hh - ll == 0 ? 0 : (((High[0] + Low[0]) / 2) - ll) / (hh - ll)));
			k_full.Set(SMA(k_fast, length2)[0]);
			d_full.Set(SMA(k_full, length3)[0]);
			
			if(k_full[0] <= 80 && k_full[0] >= 20)
			{
				MidColor.Set(1, k_full[1]);
				MidColor.Set(k_full[0]);
			}
			else 
			{
				OBOSColor.Set(1, k_full[1]);
				OBOSColor.Set(k_full[0]);
				
			}
        }

		
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MidColor
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries OBOSColor
        {
            get { return Values[1]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int Length1
        {
            get { return length1; }
            set { length1 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Length2
        {
            get { return length2; }
            set { length2 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int OverBought
        {
            get { return overBought; }
            set { overBought = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int OverSold
        {
            get { return overSold; }
            set { overSold = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BLine[] cacheBLine = null;

        private static BLine checkBLine = new BLine();

        /// <summary>
        /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
        /// </summary>
        /// <returns></returns>
        public BLine BLine(int length1, int length2, int overBought, int overSold)
        {
            return BLine(Input, length1, length2, overBought, overSold);
        }

        /// <summary>
        /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
        /// </summary>
        /// <returns></returns>
        public BLine BLine(Data.IDataSeries input, int length1, int length2, int overBought, int overSold)
        {
            checkBLine.Length1 = length1;
            length1 = checkBLine.Length1;
            checkBLine.Length2 = length2;
            length2 = checkBLine.Length2;
            checkBLine.OverBought = overBought;
            overBought = checkBLine.OverBought;
            checkBLine.OverSold = overSold;
            overSold = checkBLine.OverSold;

            if (cacheBLine != null)
                for (int idx = 0; idx < cacheBLine.Length; idx++)
                    if (cacheBLine[idx].Length1 == length1 && cacheBLine[idx].Length2 == length2 && cacheBLine[idx].OverBought == overBought && cacheBLine[idx].OverSold == overSold && cacheBLine[idx].EqualsInput(input))
                        return cacheBLine[idx];

            BLine indicator = new BLine();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Length1 = length1;
            indicator.Length2 = length2;
            indicator.OverBought = overBought;
            indicator.OverSold = overSold;
            indicator.SetUp();

            BLine[] tmp = new BLine[cacheBLine == null ? 1 : cacheBLine.Length + 1];
            if (cacheBLine != null)
                cacheBLine.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheBLine = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BLine BLine(int length1, int length2, int overBought, int overSold)
        {
            return _indicator.BLine(Input, length1, length2, overBought, overSold);
        }

        /// <summary>
        /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
        /// </summary>
        /// <returns></returns>
        public Indicator.BLine BLine(Data.IDataSeries input, int length1, int length2, int overBought, int overSold)
        {
            return _indicator.BLine(input, length1, length2, overBought, overSold);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BLine BLine(int length1, int length2, int overBought, int overSold)
        {
            return _indicator.BLine(Input, length1, length2, overBought, overSold);
        }

        /// <summary>
        /// Originally developed by Judy McKeigan (Buffy), requested by Jaap, translated into NinjaScript by NinjaTrader Technical support
        /// </summary>
        /// <returns></returns>
        public Indicator.BLine BLine(Data.IDataSeries input, int length1, int length2, int overBought, int overSold)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BLine(input, length1, length2, overBought, overSold);
        }

    }
}
#endregion
